// classes of linear algebra structures
// linear algebra functions

#ifndef __LINALG_H__
#define __LINALG_H__

#include <stdlib.h>

// class of a vector

template<class T>
class Vector {
 protected:
  int dim;
  T *items;
 public:
  Vector ();
  Vector (int n);
  Vector(const Vector<T> &v);
  ~Vector ();
  const Vector<T> &operator = (const Vector<T> &v);
  T getItem (int i);
  bool setItem (int i, T item);
  int getDim () { return dim; }
  Vector<T> operator * (const float a);
  Vector<T> operator + (const Vector<T> &v);
  bool operator == (const Vector<T> &v);
};

template<class T>
Vector<T>::Vector ()
{
  items = NULL; dim = 0;
}

template<class T>
Vector<T>::Vector (int n)
{
  items = NULL; dim = n;
  if (dim <= 0) return;
  items = new T [dim];
  if (items == NULL) return;
  for (int i = 0; i < dim; ++i) items[i] = 0;
}

template<class T>
Vector<T>::Vector(const Vector<T> &v)
{
  items = NULL; dim = v.dim;
  items = new T [dim];
  if (items == NULL) return;
  for (int i = 0; i < dim; ++i) items[i] = v.items[i];
}

template<class T>
Vector<T>::~Vector ()
{
  delete [] items; items = NULL;
  dim = 0;
}

template<class T>
const Vector<T> &Vector<T>::operator = (const Vector<T> &v)
{
  if (this == &v) return *this;
  delete [] items; items = NULL;
  dim = v.dim;
  items = new T [dim];
  if (items == NULL) return *this;
  for (int i = 0; i < dim; ++i) items[i] = v.items[i];
  return *this;
}

template<class T>
T Vector<T>::getItem (int i)
{
  if ((i >= 0) && (i < dim)) return items[i];
  return -1;
}

template<class T>
bool Vector<T>::setItem (int i, T item)
{
  if ((i >= 0) && (i < dim)) { items[i] = item; return true; }
  return false;
}

template<class T>
Vector<T> Vector<T>::operator * (const float a)
{
  Vector<T> result (dim);
  for (int i = 0; i < dim; ++i) result.items[i] = items[i] * a;
  return result;
}

template<class T>
Vector<T> Vector<T>::operator + (const Vector<T> &v)
{
  Vector<T> result (dim);
  if (dim != v.dim) return result;
  for (int i = 0; i < dim; ++i) result.items[i] = items[i] + v.items[i];
  return result;
}

template<class T>
bool Vector<T>::operator == (const Vector<T> &v)
{
  if (dim != v.dim) return false;
  for (int i = 0; i < dim; ++i) if (items[i] != v.items[i]) return false;
  return true;
}

// class of a matrix

template<class T>
class Matrix {
 protected:
  int dim_h, dim_v;
  T **items;
  T Determinant (Matrix<T> &matrix);
 public:
  Matrix ();
  Matrix (int m, int n);
  ~Matrix ();
  const Matrix<T> &operator = (const Matrix<T> &m);
  T getItem (int i, int j);
  bool setItem (int i, int j, T item);
  void getDims (int &m, int &n) { m = dim_v; n = dim_h; }
  bool isSquare () { return (dim_v == dim_h); }
  T getDeterminant () { return Determinant (*this); }
};

template<class T>
Matrix<T>::Matrix ()
{
  items = NULL; dim_v = 0; dim_h = 0;
}

template<class T>
Matrix<T>::Matrix (int m, int n)
{
  items = NULL; dim_v = m; dim_h = n;
  if ((dim_v <= 0) || (dim_h <= 0)) return;
  items = new T * [dim_v];
  if (items == NULL) return;
  for (int i = 0; i < dim_v; ++i) {
    items[i] = NULL;
    items[i] = new T [dim_h];
    if (items[i] == NULL) {
      for (int j = i - 1; j >= 0; ++j) delete [] items[j];
      delete [] items;
      return;
    }
  }
  for (int i = 0; i < dim_v; ++i) 
    for (int j = 0; j < dim_h; ++j)
      items[i][j] = 0;
}

template<class T>
Matrix<T>::~Matrix ()
{
  for (int i = 0; i < dim_v; ++i)
    delete [] items[i];
  delete [] items; items = NULL;
  dim_v = dim_h = 0;
}

template<class T>
const Matrix<T> &Matrix<T>::operator = (const Matrix<T> &m)
{
  if (this == &m) return *this;
  for (int i = 0; i < dim_v; ++i)
    delete [] items[i];
  delete [] items; items = NULL;
  dim_v = m.dim_v; dim_h = m.dim_h;
  items = new T * [dim_v];
  if (items == NULL) return NULL;
  for (int i = 0; i < dim_v; ++i) {
    items[i] = NULL;
    items[i] = new T [dim_h];
    if (items[i] == NULL) {
      for (int j = i - 1; j >= 0; ++j) delete [] items[j];
      delete [] items;
      return NULL;
    }
  }
  for (int i = 0; i < dim_v; ++i) 
    for (int j = 0; j < dim_h; ++j)
      items[i][j] = m.items[i][j];
  return *this;
}

template<class T>
T Matrix<T>::getItem (int i, int j)
{
  if ((i >= 0) && (i < dim_v) && (j >= 0) && (j < dim_h)) return items[i][j];
  return -1;
}

template<class T>
bool Matrix<T>::setItem (int i, int j, T item)
{
  if ((i >= 0) && (i < dim_v) && (j >= 0) && (j < dim_h)) {
    items[i][j] = item; return true;
  }
  return false;
}

template<class T>
T Matrix<T>::Determinant (Matrix<T> &matrix)
{
  // Laplace's spread according to the first row
  int n; int s; T a, r;
  matrix.getDims (n, n);
  if (n == 1) return matrix.getItem (0, 0);
  r = 0; s = -1;
  Matrix<T> mat (n - 1, n - 1);
  for (int i = 0; i < n; ++i) {
    for (int j = 1; j < n; ++j) // omit first row
      for (int k = 0; k < n; ++k) {
	if (k == i) continue; // omit i-th column
	a = matrix.getItem (j, k);
	if (k < i) mat.setItem (j - 1, k, a);
	else mat.setItem (j - 1, k - 1, a);
      }
    s *= -1;
    r += s * matrix.getItem (0, i) * Determinant (mat);
  }
  return r;
}

// function to solve system of equations

template<class T>
int SolveEquationSystem (Matrix<T> &matrix, Vector<T> &column,
				    Vector<T> &result)
{
  // Cramer's rule
  int m, n;
  matrix.getDims (m, n);
  T dmatrix, a;
  dmatrix = matrix.getDeterminant ();
  if (!dmatrix) return 1;
  Matrix<T> mat (n, n);
  for (int i = 0; i < n; ++i) {
    for (int j = 0; j < n; ++j)
      for (int k = 0; k < n; ++k) {
	// i-th column is vector of abs. values
	if (k == i) a = column.getItem (j);
	else a = matrix.getItem (j, k);
	mat.setItem (j, k, a);
      }
    result.setItem (i, mat.getDeterminant () / dmatrix);
  }
  return 0;
}

#endif

